﻿/******************************************************************************
Send bug reports to: philip@spigelmire.com

Copyright (c) 2010, Weatherite Corporation
All rights reserved.

Redistribution and use in source and binary forms, with or without 
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this 
list of conditions and the following disclaimer.

2. Redistributions in binary form must reproduce the above copyright notice, 
this list of conditions and the following disclaimer in the documentation 
and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE 
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

using System;

namespace Opto22Library
{
    /*
    public class O22SnapIoEventArgs : EventArgs
    {
        private int status = 0;

        public O22SnapIoEventArgs()
        {
        }

        public O22SnapIoEventArgs(int Status)
        {
            this.status = Status;
        }

        public int Status { get { return status; } set { status = value; } }
    }
*/

    public class O22SnapIoException : Exception 
    {
        private int result;
        private string message;

        public O22SnapIoException(int ResultCode)
        {
            result = ResultCode;
            SetMessage();
        }

        /* Result codes
        ERROR_BAD_TCODE = -1,
        ERROR_RESPONSE_BAD = -6,
        ERROR_NOT_CONNECTED = -9,

        // Error values generated by the I/O brain itself
        public const int BRAIN_ERROR_UNDEFINED_CMD = 0xE001;
        public const int BRAIN_ERROR_INVALID_PT_TYPE = 0xE002;
        public const int BRAIN_ERROR_INVALID_FLOAT = 0xE003;
        public const int BRAIN_ERROR_PUC_EXPECTED = 0xE004;
        public const int BRAIN_ERROR_INVALID_ADDRESS = 0xE005;
        public const int BRAIN_ERROR_INVALID_CMD_LENGTH = 0xE006;
        public const int BRAIN_ERROR_RESERVED = 0xE007;
        public const int BRAIN_ERROR_BUSY = 0xE008;
        public const int BRAIN_ERROR_CANT_ERASE_FLASH = 0xE009;
        public const int BRAIN_ERROR_CANT_PROG_FLASH = 0xE00A;
        public const int BRAIN_ERROR_IMAGE_TOO_SMALL = 0xE00B;
        public const int BRAIN_ERROR_IMAGE_CRC_MISMATCH = 0xE00C;
        public const int BRAIN_ERROR_IMAGE_LEN_MISMATCH = 0xE00D;
        public const int BRAIN_ERROR_FEATURE_NOT_IMPL = 0xE00E;
        public const int BRAIN_ERROR_WATCHDOG_TIMEOUT = 0xE00F;
        */

        //public const int ERROR_PAYLOAD_TOO_BIG = -100;

        private void SetMessage()
        {
            switch (ResultCode)
            {
                case -1:
                    message = "Transaction code from the device did not match expected value.";
                    break;
                case -6:
                    message = "Bad response from device.";
                    break;
                case -9:
                    message = "Not connected.";
                    break;
                case -100:
                    message = "Data payload is too large.  Must be 2034 bytes or less for TCP or 1480 bytes or less for UDP.";
                    break;
                case 0x0000E001:
                    message = "Device Response: Undefined command.";
                    break;
                case 0x0000E002:
                    message = "Device Response: Invalid point type.";
                    break;
                case 0x0000E003:
                    message = "Device Response: Invalid float.";
                    break;
                case 0x0000E004:
                    message = "Device Response: Power up clear expected.";
                    break;
                case 0x0000E005:
                    message = "Device Response: Invalid address.";
                    break;
                case 0x0000E006:
                    message = "Device Response: Invalid command length.";
                    break;
                case 0x0000E007:
                    message = "Device Response: Reserved.";
                    break;
                case 0x0000E008:
                    message = "Device Response: Busy.";
                    break;
                case 0x0000E009:
                    message = "Device Response: Can't erase flash.";
                    break;
                case 0x0000E00A:
                    message = "Device Response: Can't program flash.";
                    break;
                case 0x0000E00B:
                    message = "Device Response: Image too small.";
                    break;
                case 0x0000E00C:
                    message = "Device Response: Image CRC mismatch.";
                    break;
                case 0x0000E00D:
                    message = "Device Response: Image length mismatch.";
                    break;
                case 0x0000E00E:
                    message = "Device Response: Feature not implemented.";
                    break;
                case 0x0000E00F:
                    message = "Device Response: Watchdog timeout.";
                    break;
                default:
                    message = "Undefined error.";
                    break;
            }
        }

        public int ResultCode { get { return result; } }
        public override string Message { get { return message; } }

    }
}
